@@ -1,7 +1,7 @@
 /*
     Qalculate (library)
 
-    Copyright (C) 2003-2007, 2008, 2016-2018  Hanna Knutsson (hanna.knutsson@protonmail.com)
+    Copyright (C) 2003-2007, 2008, 2016-2024  Hanna Knutsson (hanna.knutsson@protonmail.com)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,7 +15,9 @@
 #include <libqalculate/includes.h>
 #include <libqalculate/Number.h>
 #include <libqalculate/QalculateDateTime.h>
-#include <sys/time.h>
+#ifndef _MSC_VER
+#	include <sys/time.h>
+#endif
 
 class QalculateDate;
 
@@ -56,6 +58,11 @@ enum {
 	MULTIPLICATION_SIGN_OPERATOR_SHORT
 };
 
+enum {
+	TAG_TYPE_HTML,
+	TAG_TYPE_TERMINAL
+};
+
 /// A structure representing a mathematical value/expression/result
 /**
 * A MathStructure can both be container representing an operation with an ordered list of children or simple value representing
@@ -143,35 +150,42 @@ enum {
 class MathStructure {
 
 	protected:
-	
+
 		size_t i_ref;
-	
+
 		StructureType m_type;
 		bool b_approx;
 		int i_precision;
-	
-		vector<MathStructure*> v_subs;
-		vector<size_t> v_order;
-		string s_sym;
+
+		std::vector<MathStructure*> v_subs;
+		std::vector<size_t> v_order;
+		std::string s_sym;
 		Number o_number;
 		Variable *o_variable;
 
 		Unit *o_unit;
 		Prefix *o_prefix;
 		bool b_plural;
-		
+
 		MathFunction *o_function;
 		MathStructure *function_value;
 		QalculateDateTime *o_datetime;
-		
+
 		ComparisonType ct_comp;
 		bool b_protected;
-		
+
 		bool b_parentheses;
-		
+
 		bool isolate_x_sub(const EvaluationOptions &eo, EvaluationOptions &eo2, const MathStructure &x_var, MathStructure *morig = NULL);
+		bool isolate_x_sub(const EvaluationOptions &eo, EvaluationOptions &eo2, const MathStructure &x_var, MathStructure *morig, size_t depth);
+		bool isolate_x(const EvaluationOptions &eo, const EvaluationOptions &feo, const MathStructure &x_var, bool check_result, size_t depth);
+
+		bool calculateFunctions(const EvaluationOptions &eo, bool recursive, bool do_unformat, size_t depth);
+
 		void init();
-	
+
+		class MathStructure_p *priv;
+
 	public:
 
 		/** @name Constructors */
@@ -183,7 +197,7 @@ class MathStructure {
 		* @param o The structure to copy.
 		*/
 		MathStructure(const MathStructure &o);
-		/** Create a new numeric structure (value=num/den*10^exp10). Equivalent to MathStructure(Number(num, den, exp10)). 
+		/** Create a new numeric structure (value=num/den*10^exp10). Equivalent to MathStructure(Number(num, den, exp10)).
 		*
 		* @param num The numerator of the numeric value.
 		* @param den The denominator of the numeric value.
@@ -196,15 +210,15 @@ class MathStructure {
 		* @param sym Symbolic/text value.
 		* @param force_symbol Do not check for undefined or date value.
 		*/
-		MathStructure(string sym, bool force_symbol = false);
+		MathStructure(std::string sym, bool force_symbol = false);
 		/** Create a new date and time structure.
 		*
-		* @param sym Date and time value.
+		* @param o_dt Date and time value.
 		*/
 		MathStructure(const QalculateDateTime &o_dt);
 		/** Create a new numeric structure with floating point value. Uses Number::setFloat().
 		*
-		* @param o Numeric value.
+		* @param float_value Numeric value.
 		*/
 		MathStructure(double float_value);
 		/** Create a new vector.
@@ -237,7 +251,7 @@ class MathStructure {
 		MathStructure(const Number &o);
 		~MathStructure();
 		//@}
-		
+
 		/** @name Functions/operators for setting type and content */
 		//@{
 		/** Set the structure to a copy of another structure. Child structures are copied.
@@ -252,7 +266,7 @@ class MathStructure {
 		* @param merge_precision Preserve the current precision (unless the new value has a lower precision).
 		*/
 		void set_nocopy(MathStructure &o, bool merge_precision = false);
-		/** Set the structure to a number (num/den*10^exp10). Equivalent to set(Number(num, den, exp10), precerve_precision). 
+		/** Set the structure to a number (num/den*10^exp10). Equivalent to set(Number(num, den, exp10), precerve_precision).
 		*
 		* @param num The numerator of the new numeric value.
 		* @param den The denominator of the new numeric value.
@@ -263,20 +277,20 @@ class MathStructure {
 		void set(long int num, long int den, long int exp10 = 0L, bool preserve_precision = false);
 		/** Set the structure to a symbolic/text value.
 		*
-		* @param o The new symolic/text value.
+		* @param sym The new symolic/text value.
 		* @param preserve_precision Preserve the current precision.
 		* @param force_symbol Do not check for undefined or date value.
 		*/
-		void set(string sym, bool preserve_precision = false, bool force_symbol = false);
+		void set(std::string sym, bool preserve_precision = false, bool force_symbol = false);
 		/** Set the structure to a date and time value.
 		*
-		* @param o The new data and time value.
+		* @param o_dt The new data and time value.
 		* @param preserve_precision Preserve the current precision.
 		*/
 		void set(const QalculateDateTime &o_dt, bool preserve_precision = false);
 		/** Set the structure to a number with a floating point value. Uses Number::setFloat().
 		*
-		* @param o The new numeric value.
+		* @param float_value The new numeric value.
 		* @param preserve_precision Preserve the current precision (unless the new value has a lower precision).
 		*/
 		void set(double float_value, bool preserve_precision = false);
@@ -349,7 +363,7 @@ class MathStructure {
 		void operator = (int i);
 		void operator = (Unit *u);
 		void operator = (Variable *v);
-		void operator = (string sym);
+		void operator = (std::string sym);
 		//@}
 
 		/** @name Functions to keep track of referrers */
@@ -368,9 +382,9 @@ class MathStructure {
 
 		/** @name Functions for symbols */
 		//@{
-		const string &symbol() const;
+		const std::string &symbol() const;
 		//@}
-		
+
 		/** @name Functions for date and time */
 		//@{
 		const QalculateDateTime *datetime() const;
@@ -388,10 +402,11 @@ class MathStructure {
 		void setPlural(bool is_plural);
 		void setUnit(Unit *u);
 		//@}
-		
+
 		/** @name Functions for mathematical functions */
 		//@{
 		void setFunction(MathFunction *f);
+		void setFunctionId(int id);
 		MathFunction *function() const;
 		const MathStructure *functionValue() const;
 		//@}
@@ -401,7 +416,7 @@ class MathStructure {
 		void setVariable(Variable *v);
 		Variable *variable() const;
 		//@}
-		
+
 		/** @name Functions for nested structures (power, muliplication, addition, vector, etc) */
 		//@{
 		/** Call this function when you have updated a child. Updates the precision.
@@ -456,8 +471,8 @@ class MathStructure {
 		//@{
 		ComparisonType comparisonType() const;
 		void setComparisonType(ComparisonType comparison_type);
-		//@}		
-		
+		//@}
+
 		/** @name Functions checking type and value */
 		//@{
 		StructureType type() const;
@@ -499,9 +514,9 @@ class MathStructure {
 		bool isApproximatelyZero() const;
 		bool isOne() const;
 		bool isMinusOne() const;
-		
+
 		bool hasNegativeSign() const;
-		
+
 		bool representsBoolean() const;
 		bool representsPositive(bool allow_units = false) const;
 		bool representsNegative(bool allow_units = false) const;
@@ -515,6 +530,7 @@ class MathStructure {
 		bool representsReal(bool allow_units = false) const;
 		bool representsNonComplex(bool allow_units = false) const;
 		bool representsComplex(bool allow_units = false) const;
+		bool representsFinite(bool allow_units = false) const;
 		bool representsNonZero(bool allow_units = false) const;
 		bool representsZero(bool allow_units = false) const;
 		bool representsApproximatelyZero(bool allow_units = false) const;
@@ -524,7 +540,7 @@ class MathStructure {
 		bool representsNonMatrix() const;
 		bool representsScalar() const;
 		//@}
-	
+
 		/** @name Functions for precision */
 		//@{
 		void setApproximate(bool is_approx = true, bool recursive = false);
@@ -533,10 +549,10 @@ class MathStructure {
 		int precision() const;
 		void mergePrecision(const MathStructure &o);
 		void mergePrecision(bool approx, int prec);
-		
+
 		//@}
 
-		/** @name Operators for structural transformations and additions 
+		/** @name Operators for structural transformations and additions
 		* These operators transforms or adds to the structure without doing any calculations
 		*/
 		//@{
@@ -550,46 +566,46 @@ class MathStructure {
 		MathStructure operator && (const MathStructure &o) const;
 		MathStructure operator || (const MathStructure &o) const;
 		MathStructure operator ! () const;
-		
+
 		void operator *= (const MathStructure &o);
 		void operator /= (const MathStructure &o);
 		void operator += (const MathStructure &o);
 		void operator -= (const MathStructure &o);
 		void operator ^= (const MathStructure &o);
-		
+
 		void operator *= (const Number &o);
 		void operator /= (const Number &o);
 		void operator += (const Number &o);
 		void operator -= (const Number &o);
 		void operator ^= (const Number &o);
-		
+
 		void operator *= (int i);
 		void operator /= (int i);
 		void operator += (int i);
 		void operator -= (int i);
 		void operator ^= (int i);
-		
+
 		void operator *= (Unit *u);
 		void operator /= (Unit *u);
 		void operator += (Unit *u);
 		void operator -= (Unit *u);
 		void operator ^= (Unit *u);
-		
+
 		void operator *= (Variable *v);
 		void operator /= (Variable *v);
 		void operator += (Variable *v);
 		void operator -= (Variable *v);
 		void operator ^= (Variable *v);
-		
-		void operator *= (string sym);
-		void operator /= (string sym);
-		void operator += (string sym);
-		void operator -= (string sym);
-		void operator ^= (string sym);
+
+		void operator *= (std::string sym);
+		void operator /= (std::string sym);
+		void operator += (std::string sym);
+		void operator -= (std::string sym);
+		void operator ^= (std::string sym);
 
 		//@}
 
-		/** @name Functions for structural transformations and additions 
+		/** @name Functions for structural transformations and additions
 		* These functions transforms or adds to the structure without doing any calculations
 		*/
 		//@{
@@ -619,17 +635,17 @@ class MathStructure {
 		void multiply(Unit *u, bool append = false);
 		void divide(Unit *u, bool append = false);
 		void raise(Unit *u);
-		void add(string sym, bool append = false);
-		void subtract(string sym, bool append = false);
-		void multiply(string sym, bool append = false);
-		void divide(string sym, bool append = false);
-		void raise(string sym);
+		void add(std::string sym, bool append = false);
+		void subtract(std::string sym, bool append = false);
+		void multiply(std::string sym, bool append = false);
+		void divide(std::string sym, bool append = false);
+		void raise(std::string sym);
 		void add_nocopy(MathStructure *o, MathOperation op, bool append = false);
 		void add_nocopy(MathStructure *o, bool append = false);
 		void subtract_nocopy(MathStructure *o, bool append = false);
 		void multiply_nocopy(MathStructure *o, bool append = false);
 		void divide_nocopy(MathStructure *o, bool append = false);
-		void raise_nocopy(MathStructure *o);		
+		void raise_nocopy(MathStructure *o);
 		void inverse();
 		void negate();
 		void setLogicalNot();
@@ -640,13 +656,14 @@ class MathStructure {
 		void transform(StructureType mtype, int i);
 		void transform(StructureType mtype, Unit *u);
 		void transform(StructureType mtype, Variable *v);
-		void transform(StructureType mtype, string sym);
+		void transform(StructureType mtype, std::string sym);
 		void transform_nocopy(StructureType mtype, MathStructure *o);
 		void transform(StructureType mtype);
 		void transform(MathFunction *o);
+		void transformById(int id);
 		void transform(ComparisonType ctype, const MathStructure &o);
 		//@}
-		
+
 		/** @name Functions/operators for comparisons */
 		//@{
 
@@ -655,11 +672,11 @@ class MathStructure {
 		bool equals(int i) const;
 		bool equals(Unit *u) const;
 		bool equals(Variable *v) const;
-		bool equals(string sym) const;
-		
+		bool equals(std::string sym) const;
+
 		ComparisonResult compare(const MathStructure &o) const;
 		ComparisonResult compareApproximately(const MathStructure &o, const EvaluationOptions &eo = default_evaluation_options) const;
-		
+
 		bool mergeInterval(const MathStructure &o, bool set_to_overlap = false);
 
 		bool operator == (const MathStructure &o) const;
@@ -667,12 +684,12 @@ class MathStructure {
 		bool operator == (int i) const;
 		bool operator == (Unit *u) const;
 		bool operator == (Variable *v) const;
-		bool operator == (string sym) const;
-		
+		bool operator == (std::string sym) const;
+
 		bool operator != (const MathStructure &o) const;
 
-		//@}		
-		
+		//@}
+
 		/** @name Functions for calculation/evaluation */
 		//@{
 		MathStructure &eval(const EvaluationOptions &eo = default_evaluation_options);
@@ -728,40 +745,43 @@ class MathStructure {
 		void setProtected(bool do_protect = true);
 		bool isProtected() const;
 		//@}
-		
+
 		/** @name Functions for format and display */
 		//@{
 		void sort(const PrintOptions &po = default_print_options, bool recursive = true);
-		bool improve_division_multipliers(const PrintOptions &po = default_print_options);
+		bool improve_division_multipliers(const PrintOptions &po = default_print_options, MathStructure *parent = NULL);
 		void setPrefixes(const PrintOptions &po = default_print_options, MathStructure *parent = NULL, size_t pindex = 0);
-		void prefixCurrencies();
+		void prefixCurrencies(const PrintOptions &po = default_print_options);
 		void format(const PrintOptions &po = default_print_options);
 		void formatsub(const PrintOptions &po = default_print_options, MathStructure *parent = NULL, size_t pindex = 0, bool recursive = true, MathStructure *top_parent = NULL);
 		void postFormatUnits(const PrintOptions &po = default_print_options, MathStructure *parent = NULL, size_t pindex = 0);
 		bool factorizeUnits();
 		void unformat(const EvaluationOptions &eo = default_evaluation_options);
 		bool needsParenthesis(const PrintOptions &po, const InternalPrintStruct &ips, const MathStructure &parent, size_t index, bool flat_division = true, bool flat_power = true) const;
+		bool removeDefaultAngleUnit(const EvaluationOptions &eo = default_evaluation_options);
 
 		int neededMultiplicationSign(const PrintOptions &po, const InternalPrintStruct &ips, const MathStructure &parent, size_t index, bool par, bool par_prev, bool flat_division = true, bool flat_power = true) const;
-		
-		string print(const PrintOptions &po = default_print_options, const InternalPrintStruct &ips = top_ips) const;
+
+		std::string print(const PrintOptions &po = default_print_options, const InternalPrintStruct &ips = top_ips) const;
+		std::string print(const PrintOptions &po, bool format, int colorize = 0, int tagtype = TAG_TYPE_HTML, const InternalPrintStruct &ips = top_ips) const;
 		//@}
-		
+
 
 		/** @name Functions for vectors */
 		//@{
 
 		MathStructure &flattenVector(MathStructure &mstruct) const;
-		
+
 		bool rankVector(bool ascending = true);
 		bool sortVector(bool ascending = true);
-		
+		void flipVector();
+
 		MathStructure &getRange(int start, int end, MathStructure &mstruct) const;
-		
+
 		void resizeVector(size_t i, const MathStructure &mfill);
 
 		//@}
-		
+
 
 		/** @name Functions for matrices */
 		//@{
@@ -791,31 +811,33 @@ class MathStructure {
 		bool invertMatrix(const EvaluationOptions &eo);
 		bool adjointMatrix(const EvaluationOptions &eo);
 		bool transposeMatrix();
-		MathStructure &cofactor(size_t r, size_t c, MathStructure &mstruct, const EvaluationOptions &eo) const;	
+		MathStructure &cofactor(size_t r, size_t c, MathStructure &mstruct, const EvaluationOptions &eo) const;
 		//@}
 
 		/** @name Functions for unit conversion */
 		//@{
 		int isUnitCompatible(const MathStructure &mstruct) const;
-		bool syncUnits(bool sync_complex_relations = false, bool *found_complex_relations = NULL, bool calculate_new_functions = false, const EvaluationOptions &feo = default_evaluation_options);
+		bool syncUnits(bool sync_nonlinear_relations = false, bool *found_nonlinear_relations = NULL, bool calculate_new_functions = false, const EvaluationOptions &feo = default_evaluation_options);
 		bool testDissolveCompositeUnit(Unit *u);
-		bool testCompositeUnit(Unit *u);	
+		bool testCompositeUnit(Unit *u);
 		bool dissolveAllCompositeUnits();
 		bool setPrefixForUnit(Unit *u, Prefix *new_prefix);
-		bool convertToBaseUnits(bool convert_complex_relations = false, bool *found_complex_relations = NULL, bool calculate_new_functions = false, const EvaluationOptions &feo = default_evaluation_options);
-		bool convert(Unit *u, bool convert_complex_relations = false, bool *found_complex_relations = NULL, bool calculate_new_functions = false, const EvaluationOptions &feo = default_evaluation_options, Prefix *new_prefix = NULL);
-		bool convert(const MathStructure unit_mstruct, bool convert_complex_relations = false, bool *found_complex_relations = NULL, bool calculate_new_functions = false, const EvaluationOptions &feo = default_evaluation_options);
+		bool convertToBaseUnits(bool convert_nonlinear_relations = false, bool *found_nonlinear_relations = NULL, bool calculate_new_functions = false, const EvaluationOptions &feo = default_evaluation_options, bool avoid_approximate_variables = false);
+		bool convert(Unit *u, bool convert_nonlinear_relations = false, bool *found_nonlinear_relations = NULL, bool calculate_new_functions = false, const EvaluationOptions &feo = default_evaluation_options, Prefix *new_prefix = NULL);
+		bool convert(const MathStructure unit_mstruct, bool convert_nonlinear_relations = false, bool *found_nonlinear_relations = NULL, bool calculate_new_functions = false, const EvaluationOptions &feo = default_evaluation_options);
 		//@}
-		
+
 		/** @name Functions for recursive search and replace */
 		//@{
 		int contains(const MathStructure &mstruct, bool structural_only = true, bool check_variables = false, bool check_functions = false, bool loose_equals = false) const;
 		size_t countOccurrences(const MathStructure &mstruct) const;
+		size_t countOccurrences(const MathStructure &mstruct, bool check_variables) const;
 		int containsRepresentativeOf(const MathStructure &mstruct, bool check_variables = false, bool check_functions = false) const;
 		int containsType(StructureType mtype, bool structural_only = true, bool check_variables = false, bool check_functions = false) const;
 		int containsRepresentativeOfType(StructureType mtype, bool check_variables = false, bool check_functions = false) const;
 		int containsFunction(MathFunction *f, bool structural_only = true, bool check_variables = false, bool check_functions = false) const;
-		int containsInterval(bool structural_only = true, bool check_variables = false, bool check_functions = false, bool ignore_high_precision_interval = false, bool include_interval_function = false) const;
+		int containsFunctionId(int id, bool structural_only = true, bool check_variables = false, bool check_functions = false) const;
+		int containsInterval(bool structural_only = true, bool check_variables = false, bool check_functions = false, int ignore_high_precision_interval = 0, bool include_interval_function = false) const;
 		int containsInfinity(bool structural_only = true, bool check_variables = false, bool check_functions = false) const;
 		bool containsOpaqueContents() const;
 		bool containsAdditionPower() const;
@@ -823,27 +845,31 @@ class MathStructure {
 		bool containsDivision() const;
 		size_t countFunctions(bool count_subfunctions = true) const;
 		void findAllUnknowns(MathStructure &unknowns_vector);
-		bool replace(const MathStructure &mfrom, const MathStructure &mto, bool once_only = false);
-		bool calculateReplace(const MathStructure &mfrom, const MathStructure &mto, const EvaluationOptions &eo);
+		bool replace(const MathStructure &mfrom, const MathStructure &mto, bool once_only = false, bool exclude_function_arguments = false);
+		bool replace(const MathStructure &mfrom, const MathStructure &mto, bool once_only, bool exclude_function_arguments, bool replace_in_variables);
+		bool replace(Variable *v, const MathStructure &mto);
+		bool calculateReplace(const MathStructure &mfrom, const MathStructure &mto, const EvaluationOptions &eo, bool exclude_function_arguments = false);
 		bool replace(const MathStructure &mfrom1, const MathStructure &mto1, const MathStructure &mfrom2, const MathStructure &mto2);
 		bool removeType(StructureType mtype);
 		//@}
-		
+
 		/** @name Functions to generate vectors for plotting */
 		//@{
 		MathStructure generateVector(MathStructure x_mstruct, const MathStructure &min, const MathStructure &max, int steps, MathStructure *x_vector = NULL, const EvaluationOptions &eo = default_evaluation_options) const;
 		MathStructure generateVector(MathStructure x_mstruct, const MathStructure &min, const MathStructure &max, const MathStructure &step, MathStructure *x_vector = NULL, const EvaluationOptions &eo = default_evaluation_options) const;
 		MathStructure generateVector(MathStructure x_mstruct, const MathStructure &x_vector, const EvaluationOptions &eo = default_evaluation_options) const;
 		//@}
-		
+
 		/** @name Differentiation and integration */
 		//@{
 		bool differentiate(const MathStructure &x_var, const EvaluationOptions &eo);
-		int integrate(const MathStructure &x_var, const EvaluationOptions &eo, bool simplify_first = true, int use_abs = 1, bool definite_integral = false, bool try_abs = true, int max_part_depth = 5, vector<MathStructure*> *parent_parts = NULL);
+		bool integrate(const MathStructure &lower_limit, const MathStructure &upper_limit, const MathStructure &x_var_pre, const EvaluationOptions &eo = default_evaluation_options, bool force_numerical = false, bool simplify_first = true);
+		int integrate(const MathStructure &x_var, const EvaluationOptions &eo, bool simplify_first = true, int use_abs = 1, bool definite_integral = false, bool try_abs = true, int max_part_depth = 5, std::vector<MathStructure*> *parent_parts = NULL);
 		//@}
 
 		/** @name Functions for polynomials */
 		//@{
+		bool expand(const EvaluationOptions &eo = default_evaluation_options, bool unfactorize = true);
 		bool simplify(const EvaluationOptions &eo = default_evaluation_options, bool unfactorize = true);
 		bool factorize(const EvaluationOptions &eo = default_evaluation_options, bool unfactorize = true, int term_combination_levels = 0, int max_msecs = 1000, bool only_integers = true, int recursive = 1, struct timeval *endtime_p = NULL, const MathStructure &force_factorization = m_undefined, bool complete_square = false, bool only_sqrfree = false, int max_degree_factor = -1);
 		bool expandPartialFractions(const EvaluationOptions &eo);
@@ -872,17 +898,18 @@ class MathStructure {
 		void polynomialPrimpart(const MathStructure &xvar, const MathStructure &c, MathStructure &mprim, const EvaluationOptions &eo) const;
 		void polynomialUnitContentPrimpart(const MathStructure &xvar, int &munit, MathStructure &mcontent, MathStructure &mprim, const EvaluationOptions &eo) const;
 		//@}
-		
+
 		/** @name Functions for conversion of complex numbers */
 		//@{
 		bool complexToExponentialForm(const EvaluationOptions &eo);
 		bool complexToPolarForm(const EvaluationOptions &eo);
+		bool complexToCisForm(const EvaluationOptions &eo);
 		//@}
-		
+
 		bool calculateLimit(const MathStructure &x_var, const MathStructure &limit, const EvaluationOptions &eo_pre, int approach_direction = 0);
 
 		bool decomposeFractions(const MathStructure &x_var, const EvaluationOptions &eo);
-		
+
 		static bool polynomialDivide(const MathStructure &mnum, const MathStructure &mden, MathStructure &mquotient, const EvaluationOptions &eo, bool check_args = true);
 		static bool polynomialQuotient(const MathStructure &mnum, const MathStructure &mden, const MathStructure &xvar, MathStructure &mquotient, const EvaluationOptions &eo, bool check_args = true);
 		static bool lcm(const MathStructure &m1, const MathStructure &m2, MathStructure &mlcm, const EvaluationOptions &eo, bool check_args = true);
@@ -894,12 +921,15 @@ class MathStructure {
 		bool isolate_x(const EvaluationOptions &eo, const MathStructure &x_var = m_undefined, bool check_result = false);
 		bool isolate_x(const EvaluationOptions &eo, const EvaluationOptions &feo, const MathStructure &x_var = m_undefined, bool check_result = false);
 		//@}
-		
+
 		bool inParentheses() const;
 		void setInParentheses(bool b = true);
 
 };
 
-ostream& operator << (ostream &os, const MathStructure&);
+std::ostream& operator << (std::ostream &os, const MathStructure&);
+
+int has_information_unit(const MathStructure &m, bool top = true);
+bool is_unit_multiexp(const MathStructure &mstruct);
 
 #endif
